
import java.io.IOException;
import java.io.InputStream;

class NetBiosSessionRequestPacket {
    static final int SESSION_MESSAGE = 0x00;
    static final int SESSION_REQUEST = 0x81;
    static final int POSITIVE_SESSION_RESPONSE = 0x82;
    static final int NEGATIVE_SESSION_RESPONSE = 0x83;
    static final int HEADER_LENGTH = 4;

    int type;
    int length;

    NetBiosName calledName;
    NetBiosName callingName;

    NetBiosSessionRequestPacket(NetBiosName calledName, NetBiosName callingName) {
        type = SESSION_REQUEST;
        this.calledName = calledName;
        this.callingName = callingName;
    }

    static void writeWord(int val, byte[] dst, int dstIndex) {
        dst[dstIndex++] = (byte) ((val >> 8) & 0xFF);
        dst[dstIndex] = (byte) (val & 0xFF);
    }

    static int readLength(byte[] src, int srcIndex) {
        srcIndex++;
        return ((src[srcIndex++] & 0x01) << 16) + ((src[srcIndex++] & 0xFF) << 8) + (src[srcIndex++] & 0xFF);
    }

    static int readPacketType(InputStream in, byte[] buffer, int bufferIndex) throws IOException {
        int n;
        if ((n = in.read(buffer, bufferIndex, HEADER_LENGTH)) != HEADER_LENGTH) {
            if (n == -1) {
                return -1;
            }
            throw new IOException("Error reading NetBios session header");
        }
        int t = buffer[bufferIndex] & 0xFF;
        return t;
    }

    int writePacket(byte[] dst, int dstIndex) throws Exception {
        length = writeNames(dst, dstIndex + HEADER_LENGTH);
        writeHeader(dst, dstIndex);
        return HEADER_LENGTH + length;
    }

    int writeHeader(byte[] dst, int dstIndex) {
        dst[dstIndex++] = (byte) type;
        if (length > 0x0000FFFF) {
            dst[dstIndex] = (byte) 0x01;
        }
        dstIndex++;
        writeWord(length, dst, dstIndex);
        return HEADER_LENGTH;
    }

    int writeNames(byte[] dst, int dstIndex) throws Exception {
        int start = dstIndex;
        dstIndex += calledName.encode(dst, dstIndex);
        dstIndex += callingName.encode(dst, dstIndex);
        return dstIndex - start;
    }

}
