
import java.net.Socket;
import java.net.InetAddress;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;


public class NetBiosSocket extends Socket {
    private static final int SSN_SRVC_PORT = 139;
    private static final int POSITIVE_SESSION_RESPONSE = 0x82;
    private static final int BUFFER_SIZE = 512;

    NetBiosName toName;
    NetBiosName fromName;

    public NetBiosSocket(NetBiosName fromName, NetBiosName toName) throws Exception {
        super(InetAddress.getByName(toName.name), SSN_SRVC_PORT);

        this.toName = toName;
        this.fromName = fromName;

        connect();
    }

    public OutputStream getOutputStream() throws IOException {
        return new NetBiosSocketOutputStream(super.getOutputStream());
    }

    private void connect() throws Exception {
        byte[] buffer = new byte[BUFFER_SIZE];

        InputStream in = super.getInputStream();
        OutputStream out = super.getOutputStream();

        NetBiosSessionRequestPacket ssp = new NetBiosSessionRequestPacket(toName, fromName);
        out.write(buffer, 0, ssp.writePacket(buffer, 0));

        int packetType = ssp.readPacketType(in, buffer, 0);
        if (packetType != POSITIVE_SESSION_RESPONSE) {
            throw new IOException("NetBios Connection error.");
        }
    }
}
