
import java.io.FilterOutputStream;
import java.io.OutputStream;
import java.io.IOException;

class NetBiosSocketOutputStream extends FilterOutputStream {
    private static int BUFFER_SIZE = 4400;

    private byte[] buffer;
    private int count;

    NetBiosSocketOutputStream(OutputStream out) {
        this(out, BUFFER_SIZE);
    }

    NetBiosSocketOutputStream(OutputStream out, int size) {
        super(out);
        buffer = new byte[size + 4];
        buffer[0] = (byte) NetBiosSessionRequestPacket.SESSION_MESSAGE;
        count = 4;
    }

    public synchronized void write(int b) throws IOException {
        if (count >= buffer.length) {
            flush();
        }
        buffer[count++] = (byte) b;
    }

    public synchronized void write(byte[] b, int off, int len) throws IOException {
        if ((len + 4) > buffer.length) {
            flush();
            byte[] tmp = buffer;
            buffer = new byte[len + 4];
            System.arraycopy(b, off, buffer, count, len);
            count += len;
            flush();
            buffer = tmp;
            return;
        }
        if (len > buffer.length - count) {
            flush();
        }
        System.arraycopy(b, off, buffer, count, len);

        count += len;
    }

    public synchronized void flush() throws IOException {
        if (count == 4) {
            return;
        }

        int n = count - 4;

        buffer[1] = (byte) ((n & 0x010000) == 0x00 ? 0x00 : 0x01);
        buffer[2] = (byte) ((n >> 8) & 0xFF);
        buffer[3] = (byte) (n & 0xFF);

        out.write(buffer, 0, count);
        out.flush();

        count = 4;
    }
}
