
import java.io.OutputStream;
import java.io.IOException;


public class SmbPacket {
    private static final byte ASCII_FORMAT = 0x04;
    private static final byte DATA_BLOCK = 0x01;

    private byte[] header = {(byte) 0xff, 0x53, 0x4d, 0x42, // Server Component
                             (byte) 0xd0, // SMB Command
                             0x00, // Error Class
                             0x00, // Reserved
                             0x00, 0x00, // Error Code
                             0x00, 0x00, 0x00, // Flags
                             0x00, 0x00, // Process ID
                             0x00, 0x00, 0x00, 0x00, // Signature ...
                             0x00, 0x00, 0x00, 0x00, // ...
                             0x00, 0x00, // Reserved
                             0x00, 0x00, // Tree ID
                             0x00, 0x00, // Process ID
                             0x00, 0x00, // User ID
                             0x00, 0x00                     // Multiplex ID
    };

    private String senderName;
    private String destinationName;
    private String message;

    public SmbPacket(String senderName, String destinationName, String message) {
        this.senderName = senderName;
        this.destinationName = destinationName;
        this.message = message;
    }

    public void send(OutputStream out) throws IOException {
        out.write(header);
        out.write(0x00);
        out.write(getBytes(senderName.length() + destinationName.length() + message.length() + 7));
        out.write(ASCII_FORMAT);
        out.write(senderName.getBytes());
        out.write(0x00);
        out.write(ASCII_FORMAT);
        out.write(destinationName.getBytes());
        out.write(0x00);
        out.write(DATA_BLOCK);
        out.write(getBytes(message.length()));
        out.write(message.getBytes());
        //
    }

    private byte[] getBytes(int number) {
        byte[] bytes = new byte[2];
        bytes[0] = (byte) (number & 0x00FF);
        bytes[1] = (byte) ((number & 0xFF00) >> 16);
        return bytes;
    }
}
